@@ -15,6 +15,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import math
 
 import mock
@@ -149,6 +150,21 @@ class StrUtilsTest(test_base.BaseTestCase):
         self.assertEqual(1, strutils.int_from_bool_as_string(True))
         self.assertEqual(0, strutils.int_from_bool_as_string(False))
 
+    def test_is_valid_boolstr(self):
+        self.assertTrue(strutils.is_valid_boolstr('true'))
+        self.assertTrue(strutils.is_valid_boolstr('false'))
+        self.assertTrue(strutils.is_valid_boolstr('yes'))
+        self.assertTrue(strutils.is_valid_boolstr('no'))
+        self.assertTrue(strutils.is_valid_boolstr('y'))
+        self.assertTrue(strutils.is_valid_boolstr('n'))
+        self.assertTrue(strutils.is_valid_boolstr('1'))
+        self.assertTrue(strutils.is_valid_boolstr('0'))
+        self.assertTrue(strutils.is_valid_boolstr(1))
+        self.assertTrue(strutils.is_valid_boolstr(0))
+
+        self.assertFalse(strutils.is_valid_boolstr('maybe'))
+        self.assertFalse(strutils.is_valid_boolstr('only on tuesdays'))
+
     def test_slugify(self):
         to_slug = strutils.to_slug
         self.assertRaises(TypeError, to_slug, True)
@@ -640,6 +656,29 @@ class MaskDictionaryPasswordTestCase(test_base.BaseTestCase):
         self.assertEqual(expected,
                          strutils.mask_dict_password(payload))
 
+    def test_other_non_str_values(self):
+        payload = {'password': 'DK0PK1AK3', 'bool': True,
+                   'dict': {'cat': 'meow', 'password': "*aa38skdjf"},
+                   'float': 0.1, 'int': 123, 'list': [1, 2], 'none': None,
+                   'str': 'foo'}
+        expected = {'password': '***', 'bool': True,
+                    'dict': {'cat': 'meow', 'password': '***'},
+                    'float': 0.1, 'int': 123, 'list': [1, 2], 'none': None,
+                    'str': 'foo'}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+    def test_argument_untouched(self):
+        """Make sure that the argument passed in is not modified"""
+        payload = {'password': 'DK0PK1AK3', 'bool': True,
+                   'dict': {'cat': 'meow', 'password': "*aa38skdjf"},
+                   'float': 0.1, 'int': 123, 'list': [1, 2], 'none': None,
+                   'str': 'foo'}
+        pristine = copy.deepcopy(payload)
+        # Send the payload into the function, to see if it gets modified
+        strutils.mask_dict_password(payload)
+        self.assertEqual(pristine, payload)
+
 
 class IsIntLikeTestCase(test_base.BaseTestCase):
     def test_is_int_like_true(self):
@@ -738,3 +777,30 @@ class SplitPathTestCase(test_base.BaseTestCase):
             strutils.split_path('o\nn e', 2, 3, True)
         except ValueError as err:
             self.assertEqual(str(err), 'Invalid path: o%0An%20e')
+
+
+class SplitByCommas(test_base.BaseTestCase):
+    def test_not_closed_quotes(self):
+        self.assertRaises(ValueError, strutils.split_by_commas, '"ab","b""')
+
+    def test_no_comma_before_opening_quotes(self):
+        self.assertRaises(ValueError, strutils.split_by_commas, '"ab""b"')
+
+    def test_quote_inside_unquoted(self):
+        self.assertRaises(ValueError, strutils.split_by_commas, 'a"b,cd')
+
+    def check(self, expect, input):
+        self.assertEqual(expect, strutils.split_by_commas(input))
+
+    def test_plain(self):
+        self.check(["a,b", "ac"], '"a,b",ac')
+
+    def test_with_backslash_inside_quoted(self):
+        self.check(['abc"', 'de', 'fg,h', 'klm\\', '"nop'],
+                   r'"abc\"","de","fg,h","klm\\","\"nop"')
+
+    def test_with_backslash_inside_unquoted(self):
+        self.check([r'a\bc', 'de'], r'a\bc,de')
+
+    def test_with_escaped_quotes_in_row_inside_quoted(self):
+        self.check(['a"b""c', 'd'], r'"a\"b\"\"c",d')
